// MainWindow.cpp: Implementierungsdatei
//

#include "stdafx.h"
#include "OpenGLMfc.h"
#include "MainWindow.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainWindow

CMainWindow::CMainWindow()
{
	leftPressed = false;
	lineSmooth = true;
	color = 1;
	strip = false;
}

CMainWindow::~CMainWindow()
{
	delete clientDC;

	Clear();
}


BEGIN_MESSAGE_MAP(CMainWindow, CWnd)
	//{{AFX_MSG_MAP(CMainWindow)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_CLOSE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_KEYDOWN()
	ON_WM_MOUSEWHEEL()
	ON_WM_RBUTTONDOWN()
	ON_WM_MBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten CMainWindow 

int CMainWindow::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	clientDC = new CClientDC(this);
	openGLDevice.create(clientDC->m_hDC);
	InitGL();

	return 0;
}

void CMainWindow::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	
	if (cy == 0)								
	{
		cy = 1;						
	}

	glViewport(0, 0, cx, cy);	

	glMatrixMode(GL_PROJECTION);						
	glLoadIdentity();						

	
	glOrtho(0,cx,0,cy,-1,1);

	glMatrixMode(GL_MODELVIEW);						
	glLoadIdentity();
}

BOOL CMainWindow::PreCreateWindow(CREATESTRUCT& cs) 
{
	screenMode.setDisplayMode(640,480,16);

	return CWnd::PreCreateWindow(cs);
}

void CMainWindow::Update()
{
	if (GetAsyncKeyState(VK_ESCAPE))
	{
		PostMessage(WM_CLOSE);
	}

	if (GetAsyncKeyState(VK_DELETE))
	{
		Clear();
	}

	DrawGLScene();
}

void CMainWindow::DrawGLScene()
{
	int arraySize = points.GetSize();
	
	glClear(GL_COLOR_BUFFER_BIT |
			GL_DEPTH_BUFFER_BIT);

	for (int i = 0; i < arraySize-1; i += 1)
	{
		if ((*points[i]) == SColorPoint(-1,-1) ||
			(*points[i+1]) == SColorPoint(-1,-1))
		{
			continue;
		}
		
		glBegin(GL_LINE_STRIP);
			points[i]->setGLColor();
			glVertex2i(points[i]->x,points[i]->y);
			glVertex2i(points[i+1]->x,points[i+1]->y);
		glEnd();
		
	}
			
	SColorPoint dummy(0,0,color);
	glDisable(GL_POLYGON_SMOOTH);

	glBegin(GL_QUADS);
		dummy.setGLColor(0.5);
		glVertex2f(0,460);
		glVertex2f(40,460);
		glVertex2f(40,480);
		glVertex2f(0,480);
	glEnd();
	glEnable(GL_POLYGON_SMOOTH);

	DrawMouse(mousePos);

	SwapBuffers(clientDC->m_hDC);
}

void CMainWindow::OnClose() 
{
	screenMode.reset();
	PostQuitMessage(0);

	CWnd::OnClose();
}

void CMainWindow::InitGL()
{
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepth(1.0f);							
	glEnable(GL_DEPTH_TEST);					
	glDepthFunc(GL_LEQUAL);	

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_LINE_SMOOTH);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);	
	glEnable(GL_POLYGON_SMOOTH);
	glLineWidth(2);
}

void CMainWindow::OnLButtonDown(UINT nFlags, CPoint point) 
{
	AddPoint(SColorPoint(point.x , point.y,color));
	leftPressed = true;

	CWnd::OnLButtonDown(nFlags, point);
}

void CMainWindow::OnLButtonUp(UINT nFlags, CPoint point) 
{
	AddPoint(SColorPoint(point.x,point.y,color));
	leftPressed = false;
	
	if (!strip)
	{
		points.Add(new SColorPoint(-1,-1));
	}

	CWnd::OnLButtonUp(nFlags, point);
}

void CMainWindow::OnMouseMove(UINT nFlags, CPoint point) 
{
	mousePos = point;

	if (leftPressed && !strip)
	{
		AddPoint(SColorPoint(point.x,point.y,color));
	}

	CWnd::OnMouseMove(nFlags, point);
}

void CMainWindow::AddPoint(SColorPoint point)
{
	const int SCREEN_HEIGHT = 480;
	int realY = SCREEN_HEIGHT - point.y;


	SColorPoint* newPoint = new SColorPoint(point.x,realY,point.color);
	points.Add(newPoint);
}

void CMainWindow::Clear()
{
	int arraySize = points.GetSize();
	for (int i = 0; i < arraySize; i += 1)
	{
		delete points[i];
	}

	points.RemoveAll();
}

void CMainWindow::DrawMouse(CPoint point)
{
	const int SCREEN_HEIGHT = 480;

	int realX = point.x;
	int realY = SCREEN_HEIGHT - point.y;
	
	glBegin(GL_TRIANGLES);
		glColor4f(1.0,1.0,1.0,0.5);
		glVertex2f(realX,realY);
		glVertex2f(realX,realY-20);
		glVertex2f(realX+20,realY);
	glEnd();

	
	

}

void CMainWindow::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch (nChar)
	{
	case 'S':
		{
			lineSmooth = !lineSmooth;

			if (lineSmooth)
			{
				glEnable(GL_LINE_SMOOTH);
			}
			else
			{
				glDisable(GL_LINE_SMOOTH);
			}
			break;
		}
	case 'M':
		{
			strip = !strip;

			if (!strip)
			{
				points.Add(new SColorPoint(-1,-1));
			}
		}

		break;
	case '1':
		color = 1;
		break;
	case '2':
		color = 2;
		break;
	case '3':
		color = 3;
		break;
	case '4':
		color = 4;
		break;
	case '5':
		color = 5;
		break;
	case '6':
		color = 6;
		break;
	case '7':
		color = 7;
		break;
	}

	points[points.GetSize()-1]->color = color;

	CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

BOOL CMainWindow::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	if (zDelta < 0)
	{
		color -= 1;

		if (color < 1)
		{
			color = 7;
		}
	}
	else if (zDelta > 0)
	{
		color += 1;

		if (color > 7)
		{
			color = 1;
		}
	}

	points[points.GetSize()-1]->color = color;

	return CWnd::OnMouseWheel(nFlags, zDelta, pt);
}

void CMainWindow::OnRButtonDown(UINT nFlags, CPoint point) 
{
	Clear();

	CWnd::OnRButtonDown(nFlags, point);
}

void CMainWindow::OnMButtonDown(UINT nFlags, CPoint point) 
{

	strip = !strip;

	if (!strip)
	{
		points.Add(new SColorPoint(-1,-1));
	}

	CWnd::OnMButtonDown(nFlags, point);
}
